;; Simple implementation of mode-line/echo-area clock, using timers.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From talos!kjones@uunet.uu.net  Tue Sep  6 11:02:59 1988
;; Received: from uunet.UU.NET by prep.ai.mit.edu; Tue, 6 Sep 88 11:02:59 EST
;; Received: from talos.UUCP by uunet.UU.NET (5.59/1.14) with UUCP 
;; 	id AA10434; Tue, 6 Sep 88 12:36:25 EDT
;; Message-Id: <8809061636.AA10434@uunet.UU.NET>
;; Date: Tue, 6 Sep 88 12:32:25 edt
;; From: talos!kjones@uunet.uu.net (Kyle Jones)
;; To: uunet!prep.ai.mit.edu!info-gnu-emacs@uunet.uu.net
;; Subject: interval timers for GNU Emacs
;; 
;; Quite a while ago I posted hooks to Emacs' lisp/time.el to support
;; time-based auto-saving.  Since then I've wished for, and finally
;; developed, a general way to schedule asynchronous events under GNU
;; Emacs.  My implementation appears below.
;; 
;; You need UNIX and Emacs subprocesses for this to work.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; From the original README file:
;;
;; Byte-compile the file timer.el and install it in a Lisp library.
;; 
;; C compile the file timer.c and install the resulting executable in one
;; of the directories in your (or Emacs') search path.  The executable
;; should be named `timer'.  Use -DUSG if running under System V.  I
;; couldn't test the code under System V.
;; 
;; A simple example of timer usage appears in the file ntime.el, which
;; contains a no-frills rewrite of Emacs' time.el.
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(require 'timer)

(defvar display-time-interval 60)
(defvar display-time-echo-area nil)
(defvar display-time-day-and-date nil)

(defun display-time ()
  (interactive)
  (if (get-timer "display-time")
      ()
    (or display-time-echo-area
	global-mode-string
	(setq global-mode-string '("")))
    (or display-time-echo-area
	(memq 'display-time-string global-mode-string)
	(setq global-mode-string
	      (append global-mode-string '(display-time-string))))
    (display-time-function)
    (start-timer "display-time" 'display-time-function
		 display-time-interval display-time-interval)))

(defun display-time-function ()
  (let (string)
    (setq string (substring (current-time-string)
			    (if display-time-day-and-date 0 11) -8))
    (if display-time-echo-area
	(if (zerop (minibuffer-depth))
	    (save-excursion
	      (set-buffer (window-buffer (minibuffer-window)))
	      (erase-buffer)
	      (indent-to (- (screen-width) (length string) 1))
	      (insert string)))
      (setq display-time-string string)
      (save-excursion (set-buffer (other-buffer)))
      (set-buffer-modified-p (buffer-modified-p)))))
